//
// etimeget.cc
//
// Authors:  Richard Saxton (VEGA/ESA)
//           ESA XMM SOC
//
// Description: Metatask which runs evselect, arfgen and rmfgen to 
//              produce source and background timetra and the appropriate
//              ARF and RMF.
//
//#include <set>          // C++
//#include <algorithm>
//#include <strstream.h>
//#include <unistd.h>
#include <Tracer.h>
#include <metatask.h>
#include <errstr.h>
#include <param.h>
#include "Evaluator.h"


#include "etimeget.h"
#include "DatafileAttributes.h"
#include "LightCurveMaker.h"

void etimeget()
{
  PROC;

  /* Set the source, backgnd and arf filenames */
    string sourceset, backset, arfset, outrmfset;

    if  (booleanParameter("withfilestem")) {
        string filestem = stringParameter("filestem");
        sourceset = filestem+"_srcrate.lc";
        backset   = filestem+"_bgdrate.lc";
    } else {
        sourceset = stringParameter("srcrateset");
        backset   = stringParameter("bckrateset");
    }

  /* Get the parameters from this command line */
    string srcexp = stringParameter("srcexp");
    string backexp = stringParameter("backexp");
    string eventfile = stringParameter("table");
    string timebinstyle = stringParameter("timebinstyle");
    double timebin = realParameter("timebinwidth");

  /* Check that the input expressions are valid */
    if (! Evaluator::isValidExpression(srcexp)) {
       errstr<<"Invalid expression: "<<srcexp
             << Sas::error("InvalidExpression");
    }
    else if (! Evaluator::isValidExpression(backexp)) {
       errstr<<"Invalid expression: "<<backexp
             << Sas::error("InvalidExpression");
    }

  /*----------------------------------------------------------
     Set the timebin size for the lightcurve
  ----------------------------------------------------------*/

   /* Read observation attributes from the event file */
    DatafileAttributes * timeAttributes = new DatafileAttributes(eventfile);

   /* If timebinstyle=intrinsic, find the intrinsic bin width for
      this instrument and observing mode */

    if (timebinstyle == "intrinsic") {
       timebin = timeAttributes->intrinsicFrameTime();
       errstr<<"Intrinsic frame time is "<<timebin
          << Sas::message(Msg::App, Msg::Sparse);

    }
    else if (timebinstyle == "parameter") 
    {
       // timebin set earlier
    }
    else {   // Bin style not recognised
       errstr<<"Invalid time bin style: "<<timebinstyle
             << Sas::error("InvalidTimebinstyle");
    }

  /* Create the source time series */
    errstr<<"Creating source time series..."
          << Sas::message(Msg::App, Msg::Sparse);

    LightCurveMaker stime = LightCurveMaker(sourceset, timeAttributes,
                                                   srcexp, timebin);

  /* Create the background timetrum */
    errstr<<"Creating background time series..."
          << Sas::message(Msg::App, Msg::Sparse);

    LightCurveMaker btime = LightCurveMaker(backset, timeAttributes,
                                                  backexp, timebin);
  /* Shutdown */
    delete timeAttributes; timeAttributes=0;

    closeParameters();

  ENDPROC;
}
